@@ -1,5 +1,5 @@
 /*
-  Copyright 2007-2014 David Robillard <http://drobilla.net>
+  Copyright 2007-2016 David Robillard <http://drobilla.net>
 
   Permission to use, copy, modify, and/or distribute this software for any
   purpose with or without fee is hereby granted, provided that the above
@@ -43,7 +43,7 @@ extern "C" {
 #ifndef NAN
 #    define NAN INFINITY - INFINITY
 #endif
-static inline char* dlerror(void) { return "Unknown error"; }
+static inline const char* dlerror(void) { return "Unknown error"; }
 #else
 #    include <dlfcn.h>
 #    include <unistd.h>
@@ -127,6 +127,7 @@ struct LilvPluginImpl {
 	LilvPort**             ports;
 	uint32_t               num_ports;
 	bool                   loaded;
+	bool                   parse_errors;
 	bool                   replaced;
 };
 
@@ -156,6 +157,7 @@ struct LilvWorldImpl {
 	LilvPluginClasses* plugin_classes;
 	LilvSpec*          specs;
 	LilvPlugins*       plugins;
+	LilvPlugins*       zombies;
 	LilvNodes*         loaded_files;
 	ZixTree*           libs;
 	struct {
@@ -172,7 +174,9 @@ struct LilvWorldImpl {
 		SordNode* lv2_index;
 		SordNode* lv2_latency;
 		SordNode* lv2_maximum;
+		SordNode* lv2_microVersion;
 		SordNode* lv2_minimum;
+		SordNode* lv2_minorVersion;
 		SordNode* lv2_name;
 		SordNode* lv2_optionalFeature;
 		SordNode* lv2_port;
@@ -233,6 +237,11 @@ struct LilvUIImpl {
 	LilvNodes* classes;
 };
 
+typedef struct LilvVersion {
+	int minor;
+	int micro;
+} LilvVersion;
+
 /*
  *
  * Functions
@@ -248,6 +257,7 @@ void      lilv_port_free(const LilvPlugin* plugin, LilvPort* port);
 LilvPlugin* lilv_plugin_new(LilvWorld* world,
                             LilvNode*  uri,
                             LilvNode*  bundle_uri);
+void        lilv_plugin_clear(LilvPlugin* plugin, LilvNode* bundle_uri);
 void        lilv_plugin_load_if_necessary(const LilvPlugin* p);
 void        lilv_plugin_free(LilvPlugin* plugin);
 LilvNode*   lilv_plugin_get_unique(const LilvPlugin* p,
@@ -282,7 +292,8 @@ LilvScalePoints*   lilv_scale_points_new(void);
 LilvPluginClasses* lilv_plugin_classes_new(void);
 LilvUIs*           lilv_uis_new(void);
 
-LilvNode* lilv_world_get_manifest_uri(LilvWorld* world, LilvNode* bundle_uri);
+LilvNode* lilv_world_get_manifest_uri(LilvWorld*      world,
+                                      const LilvNode* bundle_uri);
 
 const uint8_t* lilv_world_blank_node_prefix(LilvWorld* world);
 
@@ -311,6 +322,19 @@ int lilv_lib_compare(const void* a, const void* b, void* user_data);
 int lilv_ptr_cmp(const void* a, const void* b, void* user_data);
 int lilv_resource_node_cmp(const void* a, const void* b, void* user_data);
 
+static inline int
+lilv_version_cmp(const LilvVersion* a, const LilvVersion* b)
+{
+	if (a->minor == b->minor && a->micro == b->micro) {
+		return 0;
+	} else if ((a->minor < b->minor)
+	           || (a->minor == b->minor && a->micro < b->micro)) {
+		return -1;
+	} else {
+		return 1;
+	}
+}
+
 struct LilvHeader*
 lilv_collection_get_by_uri(const ZixTree* seq, const LilvNode* uri);
 
@@ -405,6 +429,10 @@ static const LV2_Feature* const dman_features = { NULL };
                                       __func__)
 #define LILV_WARNF(fmt, ...)  fprintf(stderr, "%s(): warning: " fmt, \
                                       __func__, __VA_ARGS__)
+#define LILV_NOTE(str)        fprintf(stderr, "%s(): note: " str, \
+                                      __func__)
+#define LILV_NOTEF(fmt, ...)  fprintf(stderr, "%s(): note: " fmt, \
+                                      __func__, __VA_ARGS__)
 
 #ifdef __cplusplus
 }
