@@ -1,5 +1,5 @@
 /*
-  Copyright 2007-2015 David Robillard <http://drobilla.net>
+  Copyright 2007-2016 David Robillard <http://drobilla.net>
 
   Permission to use, copy, modify, and/or distribute this software for any
   purpose with or without fee is hereby granted, provided that the above
@@ -15,16 +15,16 @@
 */
 
 #include <math.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
 #include "lilv_internal.h"
 
 static void
-lilv_node_set_numerics_from_string(LilvNode* val, size_t len)
+lilv_node_set_numerics_from_string(LilvNode* val)
 {
 	const char* str = (const char*)sord_node_get_string(val->node);
-	char*       endptr;
 
 	switch (val->type) {
 	case LILV_VALUE_URI:
@@ -33,10 +33,10 @@ lilv_node_set_numerics_from_string(LilvNode* val, size_t len)
 	case LILV_VALUE_BLOB:
 		break;
 	case LILV_VALUE_INT:
-		val->val.int_val = strtol(str, &endptr, 10);
+		val->val.int_val = strtol(str, NULL, 10);
 		break;
 	case LILV_VALUE_FLOAT:
-		val->val.float_val = serd_strtod(str, &endptr);
+		val->val.float_val = serd_strtod(str, NULL);
 		break;
 	case LILV_VALUE_BOOL:
 		val->val.bool_val = !strcmp(str, "true");
@@ -104,7 +104,6 @@ lilv_node_new_from_node(LilvWorld* world, const SordNode* node)
 	LilvNode*    result       = NULL;
 	SordNode*    datatype_uri = NULL;
 	LilvNodeType type         = LILV_VALUE_STRING;
-	size_t       len          = 0;
 
 	switch (sord_node_get_type(node)) {
 	case SORD_URI:
@@ -137,8 +136,8 @@ lilv_node_new_from_node(LilvWorld* world, const SordNode* node)
 				            sord_node_get_string(datatype_uri));
 		}
 		result = lilv_node_new(
-			world, type, (const char*)sord_node_get_string_counted(node, &len));
-		lilv_node_set_numerics_from_string(result, len);
+			world, type, (const char*)sord_node_get_string(node));
+		lilv_node_set_numerics_from_string(result);
 		break;
 	}
 
