@@ -1,5 +1,5 @@
 /*
-  Copyright 2007-2014 David Robillard <http://drobilla.net>
+  Copyright 2007-2016 David Robillard <http://drobilla.net>
 
   Permission to use, copy, modify, and/or distribute this software for any
   purpose with or without fee is hereby granted, provided that the above
@@ -32,14 +32,9 @@
 #define NS_DOAP (const uint8_t*)"http://usefulinc.com/ns/doap#"
 #define NS_FOAF (const uint8_t*)"http://xmlns.com/foaf/0.1/"
 
-/** Ownership of `uri` is taken */
-LilvPlugin*
-lilv_plugin_new(LilvWorld* world, LilvNode* uri, LilvNode* bundle_uri)
+static void
+lilv_plugin_init(LilvPlugin* plugin, LilvNode* bundle_uri)
 {
-	assert(bundle_uri);
-	LilvPlugin* plugin = (LilvPlugin*)malloc(sizeof(LilvPlugin));
-	plugin->world        = world;
-	plugin->plugin_uri   = uri;
 	plugin->bundle_uri   = bundle_uri;
 	plugin->binary_uri   = NULL;
 #ifdef LILV_DYN_MANIFEST
@@ -50,11 +45,32 @@ lilv_plugin_new(LilvWorld* world, LilvNode* uri, LilvNode* bundle_uri)
 	plugin->ports        = NULL;
 	plugin->num_ports    = 0;
 	plugin->loaded       = false;
+	plugin->parse_errors = false;
 	plugin->replaced     = false;
+}
 
+/** Ownership of `uri` and `bundle` is taken */
+LilvPlugin*
+lilv_plugin_new(LilvWorld* world, LilvNode* uri, LilvNode* bundle_uri)
+{
+	LilvPlugin* plugin = (LilvPlugin*)malloc(sizeof(LilvPlugin));
+
+	plugin->world      = world;
+	plugin->plugin_uri = uri;
+
+	lilv_plugin_init(plugin, bundle_uri);
 	return plugin;
 }
 
+void
+lilv_plugin_clear(LilvPlugin* plugin, LilvNode* bundle_uri)
+{
+	lilv_node_free(plugin->bundle_uri);
+	lilv_node_free(plugin->binary_uri);
+	lilv_nodes_free(plugin->data_uris);
+	lilv_plugin_init(plugin, bundle_uri);
+}
+
 static void
 lilv_plugin_free_ports(LilvPlugin* p)
 {
@@ -112,7 +128,8 @@ lilv_plugin_get_one(const LilvPlugin* p,
 	SordIter* stream = lilv_world_query_internal(
 		p->world, subject, predicate, NULL);
 	if (!sord_iter_end(stream)) {
-		ret = lilv_node_new_from_node(p->world, sord_iter_get_node(stream, SORD_OBJECT));
+		ret = lilv_node_new_from_node(p->world,
+		                              sord_iter_get_node(stream, SORD_OBJECT));
 	}
 	sord_iter_free(stream);
 	return ret;
@@ -125,7 +142,7 @@ lilv_plugin_get_unique(const LilvPlugin* p,
 {
 	LilvNode* ret = lilv_plugin_get_one(p, subject, predicate);
 	if (!ret) {
-		LILV_ERRORF("Multiple values found for (%s %s ...) property\n",
+		LILV_ERRORF("No value found for (%s %s ...) property\n",
 		            sord_node_get_string(subject),
 		            sord_node_get_string(predicate));
 	}
@@ -142,13 +159,13 @@ lilv_plugin_load(LilvPlugin* p)
 	SerdReader* reader = sord_new_reader(p->world->model, env, SERD_TURTLE,
 	                                     bundle_uri_node);
 
-	SordModel* prototypes = lilv_world_filter_model(p->world,
-	                                                p->world->model,
-	                                                p->plugin_uri->node,
-	                                                p->world->uris.lv2_prototype,
-	                                                NULL, NULL);
+	SordModel* prots = lilv_world_filter_model(p->world,
+	                                           p->world->model,
+	                                           p->plugin_uri->node,
+	                                           p->world->uris.lv2_prototype,
+	                                           NULL, NULL);
 	SordModel* skel = sord_new(p->world->world, SORD_SPO, false);
-	SordIter*  iter = sord_begin(prototypes);
+	SordIter*  iter = sord_begin(prots);
 	for (; !sord_iter_end(iter); sord_iter_next(iter)) {
 		const SordNode* t         = sord_iter_get_node(iter, SORD_OBJECT);
 		LilvNode*       prototype = lilv_node_new_from_node(p->world, t);
@@ -176,14 +193,26 @@ lilv_plugin_load(LilvPlugin* p)
 	}
 	sord_iter_free(iter);
 	sord_free(skel);
-	sord_free(prototypes);
+	sord_free(prots);
 
 	// Parse all the plugin's data files into RDF model
+	SerdStatus st = SERD_SUCCESS;
 	LILV_FOREACH(nodes, i, p->data_uris) {
 		const LilvNode* data_uri = lilv_nodes_get(p->data_uris, i);
 
 		serd_env_set_base_uri(env, sord_node_to_serd_node(data_uri->node));
-		lilv_world_load_file(p->world, reader, data_uri);
+		st = lilv_world_load_file(p->world, reader, data_uri);
+		if (st > SERD_FAILURE) {
+			break;
+		}
+	}
+
+	if (st > SERD_FAILURE) {
+		p->loaded       = true;
+		p->parse_errors = true;
+		serd_reader_free(reader);
+		serd_env_free(env);
+		return;
 	}
 
 #ifdef LILV_DYN_MANIFEST
@@ -234,8 +263,8 @@ static void
 lilv_plugin_load_ports_if_necessary(const LilvPlugin* const_p)
 {
 	LilvPlugin* p = (LilvPlugin*)const_p;
-	if (!p->loaded)
-		lilv_plugin_load(p);
+
+	lilv_plugin_load_if_necessary(p);
 
 	if (!p->ports) {
 		p->ports = (LilvPort**)malloc(sizeof(LilvPort*));
@@ -254,21 +283,24 @@ lilv_plugin_load_ports_if_necessary(const LilvPlugin* const_p)
 			LilvNode* symbol = lilv_plugin_get_unique(
 				p, port, p->world->uris.lv2_symbol);
 
-			bool error = false;
 			if (!lilv_node_is_string(symbol) ||
 			    !is_symbol((const char*)sord_node_get_string(symbol->node))) {
 				LILV_ERRORF("Plugin <%s> port symbol `%s' is invalid\n",
 				            lilv_node_as_uri(p->plugin_uri),
 				            lilv_node_as_string(symbol));
-				error = true;
-				goto done;
+				lilv_node_free(symbol);
+				lilv_node_free(index);
+				lilv_plugin_free_ports(p);
+				break;
 			}
 
 			if (!lilv_node_is_int(index)) {
 				LILV_ERRORF("Plugin <%s> port index is not an integer\n",
 				            lilv_node_as_uri(p->plugin_uri));
-				error = true;
-				goto done;
+				lilv_node_free(symbol);
+				lilv_node_free(index);
+				lilv_plugin_free_ports(p);
+				break;
 			}
 
 			uint32_t  this_index = lilv_node_as_int(index);
@@ -307,13 +339,8 @@ lilv_plugin_load_ports_if_necessary(const LilvPlugin* const_p)
 			}
 			sord_iter_free(types);
 
-		done:
 			lilv_node_free(symbol);
 			lilv_node_free(index);
-			if (error) {  // Invalid plugin
-				lilv_plugin_free_ports(p);
-				break;
-			}
 		}
 		sord_iter_free(ports);
 
@@ -388,12 +415,12 @@ lilv_plugin_get_class(const LilvPlugin* const_p)
 	lilv_plugin_load_if_necessary(p);
 	if (!p->plugin_class) {
 		// <plugin> a ?class
-		SordIter* results = lilv_world_query_internal(p->world,
-		                                              p->plugin_uri->node,
-		                                              p->world->uris.rdf_a,
-		                                              NULL);
-		FOREACH_MATCH(results) {
-			const SordNode* class_node = sord_iter_get_node(results, SORD_OBJECT);
+		SordIter* c = lilv_world_query_internal(p->world,
+		                                        p->plugin_uri->node,
+		                                        p->world->uris.rdf_a,
+		                                        NULL);
+		FOREACH_MATCH(c) {
+			const SordNode* class_node = sord_iter_get_node(c, SORD_OBJECT);
 			if (sord_node_get_type(class_node) != SORD_URI) {
 				continue;
 			}
@@ -412,7 +439,7 @@ lilv_plugin_get_class(const LilvPlugin* const_p)
 
 			lilv_node_free(klass);
 		}
-		sord_iter_free(results);
+		sord_iter_free(c);
 
 		if (p->plugin_class == NULL)
 			p->plugin_class = p->world->lv2_plugin_class;
@@ -432,6 +459,11 @@ lilv_plugin_get_value_internal(const LilvPlugin* p,
 LILV_API bool
 lilv_plugin_verify(const LilvPlugin* plugin)
 {
+	lilv_plugin_load_if_necessary(plugin);
+	if (plugin->parse_errors) {
+		return false;
+	}
+
 	LilvNode*  rdf_type = lilv_new_uri(plugin->world, LILV_NS_RDF "type");
 	LilvNodes* results  = lilv_plugin_get_value(plugin, rdf_type);
 	lilv_node_free(rdf_type);
@@ -866,46 +898,36 @@ lilv_plugin_get_author(const LilvPlugin* p)
 	return author;
 }
 
-LILV_API LilvNode*
-lilv_plugin_get_author_name(const LilvPlugin* plugin)
+static LilvNode*
+lilv_plugin_get_author_property(const LilvPlugin* plugin, const uint8_t* uri)
 {
 	const SordNode* author = lilv_plugin_get_author(plugin);
 	if (author) {
-		SordWorld* sworld    = plugin->world->world;
-		SordNode*  foaf_name = sord_new_uri(sworld, NS_FOAF "name");
-		LilvNode*  ret       = lilv_plugin_get_one(plugin, author, foaf_name);
-		sord_node_free(sworld, foaf_name);
+		SordWorld* sworld = plugin->world->world;
+		SordNode*  pred   = sord_new_uri(sworld, uri);
+		LilvNode*  ret    = lilv_plugin_get_one(plugin, author, pred);
+		sord_node_free(sworld, pred);
 		return ret;
 	}
 	return NULL;
 }
 
+LILV_API LilvNode*
+lilv_plugin_get_author_name(const LilvPlugin* plugin)
+{
+	return lilv_plugin_get_author_property(plugin, NS_FOAF "name");
+}
+
 LILV_API LilvNode*
 lilv_plugin_get_author_email(const LilvPlugin* plugin)
 {
-	const SordNode* author = lilv_plugin_get_author(plugin);
-	if (author) {
-		SordWorld* sworld    = plugin->world->world;
-		SordNode*  foaf_mbox = sord_new_uri(sworld, NS_FOAF "mbox");
-		LilvNode*  ret       = lilv_plugin_get_one(plugin, author, foaf_mbox);
-		sord_node_free(sworld, foaf_mbox);
-		return ret;
-	}
-	return NULL;
+	return lilv_plugin_get_author_property(plugin, NS_FOAF "mbox");
 }
 
 LILV_API LilvNode*
 lilv_plugin_get_author_homepage(const LilvPlugin* plugin)
 {
-	const SordNode* author = lilv_plugin_get_author(plugin);
-	if (author) {
-		SordWorld* sworld        = plugin->world->world;
-		SordNode*  foaf_homepage = sord_new_uri(sworld, NS_FOAF "homepage");
-		LilvNode*  ret           = lilv_plugin_get_one(plugin, author, foaf_homepage);
-		sord_node_free(sworld, foaf_homepage);
-		return ret;
-	}
-	return NULL;
+	return lilv_plugin_get_author_property(plugin, NS_FOAF "homepage");
 }
 
 LILV_API bool
@@ -934,7 +956,10 @@ lilv_plugin_get_uis(const LilvPlugin* p)
 		const SordNode* ui = sord_iter_get_node(uis, SORD_OBJECT);
 
 		LilvNode* type   = lilv_plugin_get_unique(p, ui, p->world->uris.rdf_a);
-		LilvNode* binary = lilv_plugin_get_unique(p, ui, ui_binary_node);
+		LilvNode* binary = lilv_plugin_get_one(p, ui, p->world->uris.lv2_binary);
+		if (!binary) {
+			binary = lilv_plugin_get_unique(p, ui, ui_binary_node);
+		}
 
 		if (sord_node_get_type(ui) != SORD_URI
 		    || !lilv_node_is_uri(type)
