@@ -1,5 +1,5 @@
 /*
-  Copyright 2007-2014 David Robillard <http://drobilla.net>
+  Copyright 2007-2016 David Robillard <http://drobilla.net>
 
   Permission to use, copy, modify, and/or distribute this software for any
   purpose with or without fee is hereby granted, provided that the above
@@ -23,6 +23,9 @@
 
 #include "lilv_internal.h"
 
+static int
+lilv_world_drop_graph(LilvWorld* world, const SordNode* graph);
+
 LILV_API LilvWorld*
 lilv_world_new(void)
 {
@@ -39,6 +42,7 @@ lilv_world_new(void)
 	world->specs          = NULL;
 	world->plugin_classes = lilv_plugin_classes_new();
 	world->plugins        = lilv_plugins_new();
+	world->zombies        = lilv_plugins_new();
 	world->loaded_files   = zix_tree_new(
 		false, lilv_resource_node_cmp, NULL, (ZixDestroyFunc)lilv_node_free);
 
@@ -63,7 +67,9 @@ lilv_world_new(void)
 	world->uris.lv2_index           = NEW_URI(LV2_CORE__index);
 	world->uris.lv2_latency         = NEW_URI(LV2_CORE__latency);
 	world->uris.lv2_maximum         = NEW_URI(LV2_CORE__maximum);
+	world->uris.lv2_microVersion    = NEW_URI(LV2_CORE__microVersion);
 	world->uris.lv2_minimum         = NEW_URI(LV2_CORE__minimum);
+	world->uris.lv2_minorVersion    = NEW_URI(LV2_CORE__minorVersion);
 	world->uris.lv2_name            = NEW_URI(LV2_CORE__name);
 	world->uris.lv2_optionalFeature = NEW_URI(LV2_CORE__optionalFeature);
 	world->uris.lv2_port            = NEW_URI(LV2_CORE__port);
@@ -133,6 +139,13 @@ lilv_world_free(LilvWorld* world)
 	zix_tree_free((ZixTree*)world->plugins);
 	world->plugins = NULL;
 
+	LILV_FOREACH(plugins, i, world->zombies) {
+		const LilvPlugin* p = lilv_plugins_get(world->zombies, i);
+		lilv_plugin_free((LilvPlugin*)p);
+	}
+	zix_tree_free((ZixTree*)world->zombies);
+	world->zombies = NULL;
+
 	zix_tree_free((ZixTree*)world->loaded_files);
 	world->loaded_files = NULL;
 
@@ -180,6 +193,9 @@ lilv_world_find_nodes(LilvWorld*      world,
 		LILV_ERRORF("Subject `%s' is not a resource\n",
 		            sord_node_get_string(subject->node));
 		return NULL;
+	} else if (!predicate) {
+		LILV_ERROR("Missing required predicate\n");
+		return NULL;
 	} else if (!lilv_node_is_uri(predicate)) {
 		LILV_ERRORF("Predicate `%s' is not a URI\n",
 		            sord_node_get_string(predicate->node));
@@ -278,12 +294,8 @@ lilv_new_uri_relative_to_base(const uint8_t* uri_str,
                               const uint8_t* base_uri_str)
 {
 	SerdURI base_uri;
-	if (serd_uri_parse(base_uri_str, &base_uri)) {
-		return SERD_NODE_NULL;
-	}
-
-	SerdURI ignored;
-	return serd_node_new_uri_from_string(uri_str, &base_uri, &ignored);
+	serd_uri_parse(base_uri_str, &base_uri);
+	return serd_node_new_uri_from_string(uri_str, &base_uri, NULL);
 }
 
 const uint8_t*
@@ -322,23 +334,24 @@ lilv_lib_compare(const void* a, const void* b, void* user_data)
 }
 
 /** Get an element of a collection of any object with an LilvHeader by URI. */
-struct LilvHeader*
-lilv_collection_get_by_uri(const ZixTree*  const_seq,
-                           const LilvNode* uri)
+static ZixTreeIter*
+lilv_collection_find_by_uri(const ZixTree* seq, const LilvNode* uri)
 {
-	if (!lilv_node_is_uri(uri)) {
-		return NULL;
+	ZixTreeIter* i = NULL;
+	if (lilv_node_is_uri(uri)) {
+		struct LilvHeader key = { NULL, (LilvNode*)uri };
+		zix_tree_find(seq, &key, &i);
 	}
+	return i;
+}
 
-	ZixTree*          seq = (ZixTree*)const_seq;
-	struct LilvHeader key = { NULL, (LilvNode*)uri };
-	ZixTreeIter*      i   = NULL;
-	ZixStatus         st  = zix_tree_find(seq, &key, &i);
-	if (!st) {
-		return (struct LilvHeader*)zix_tree_get(i);
-	}
+/** Get an element of a collection of any object with an LilvHeader by URI. */
+struct LilvHeader*
+lilv_collection_get_by_uri(const ZixTree* seq, const LilvNode* uri)
+{
+	ZixTreeIter* const i = lilv_collection_find_by_uri(seq, uri);
 
-	return NULL;
+	return i ? (struct LilvHeader*)zix_tree_get(i) : NULL;
 }
 
 static void
@@ -371,33 +384,55 @@ lilv_world_add_spec(LilvWorld*      world,
 }
 
 static void
-lilv_world_add_plugin(LilvWorld*       world,
-                      const SordNode*  plugin_node,
-                      const LilvNode*  manifest_uri,
-                      void*            dynmanifest,
-                      const SordNode*  bundle_node)
+lilv_world_add_plugin(LilvWorld*      world,
+                      const SordNode* plugin_node,
+                      const LilvNode* manifest_uri,
+                      void*           dynmanifest,
+                      const SordNode* bundle)
 {
-	LilvNode* plugin_uri = lilv_node_new_from_node(world, plugin_node);
+	LilvNode*    plugin_uri = lilv_node_new_from_node(world, plugin_node);
+	ZixTreeIter* z          = NULL;
+	LilvPlugin*  plugin     = (LilvPlugin*)lilv_plugins_get_by_uri(
+		world->plugins, plugin_uri);
 
-	const LilvPlugin* last = lilv_plugins_get_by_uri(world->plugins,
-	                                                 plugin_uri);
-	if (last) {
-		LILV_ERRORF("Duplicate plugin <%s>\n", lilv_node_as_uri(plugin_uri));
-		LILV_ERRORF("... found in %s\n", lilv_node_as_string(
-			            lilv_plugin_get_bundle_uri(last)));
-		LILV_ERRORF("... and      %s\n", sord_node_get_string(bundle_node));
+	if (plugin) {
+		// Existing plugin, if this is different bundle, ignore it
+		// (use the first plugin found in LV2_PATH)
+		const LilvNode* last_bundle    = lilv_plugin_get_bundle_uri(plugin);
+		const char*     plugin_uri_str = lilv_node_as_uri(plugin_uri);
+		if (sord_node_equals(bundle, last_bundle->node)) {
+			LILV_WARNF("Reloading plugin <%s>\n", plugin_uri_str);
+			plugin->loaded = false;
+			lilv_node_free(plugin_uri);
+		} else {
+			LILV_WARNF("Duplicate plugin <%s>\n", plugin_uri_str);
+			LILV_WARNF("... found in %s\n", lilv_node_as_string(last_bundle));
+			LILV_WARNF("... and      %s (ignored)\n", sord_node_get_string(bundle));
+			lilv_node_free(plugin_uri);
+			return;
+		}
+	} else if ((z = lilv_collection_find_by_uri((const ZixTree*)world->zombies,
+	                                            plugin_uri))) {
+		// Plugin bundle has been re-loaded, move from zombies to plugins
+		plugin = (LilvPlugin*)zix_tree_get(z);
+		zix_tree_remove((ZixTree*)world->zombies, z);
+		zix_tree_insert((ZixTree*)world->plugins, plugin, NULL);
 		lilv_node_free(plugin_uri);
-		return;
+		lilv_plugin_clear(plugin, lilv_node_new_from_node(world, bundle));
+	} else {
+		// Add new plugin to the world
+		plugin = lilv_plugin_new(
+			world, plugin_uri, lilv_node_new_from_node(world, bundle));
+
+		// Add manifest as plugin data file (as if it were rdfs:seeAlso)
+		zix_tree_insert((ZixTree*)plugin->data_uris,
+		                lilv_node_duplicate(manifest_uri),
+		                NULL);
+
+		// Add plugin to world plugin sequence
+		zix_tree_insert((ZixTree*)world->plugins, plugin, NULL);
 	}
 
-	// Create LilvPlugin
-	LilvNode*   bundle_uri = lilv_node_new_from_node(world, bundle_node);
-	LilvPlugin* plugin     = lilv_plugin_new(world, plugin_uri, bundle_uri);
-
-	// Add manifest as plugin data file (as if it were rdfs:seeAlso)
-	zix_tree_insert((ZixTree*)plugin->data_uris,
-	                lilv_node_duplicate(manifest_uri),
-	                NULL);
 
 #ifdef LILV_DYN_MANIFEST
 	// Set dynamic manifest library URI, if applicable
@@ -420,9 +455,6 @@ lilv_world_add_plugin(LilvWorld*       world,
 		                NULL);
 	}
 	sord_iter_free(files);
-
-	// Add plugin to world plugin sequence
-	zix_tree_insert((ZixTree*)world->plugins, plugin, NULL);
 }
 
 SerdStatus
@@ -480,7 +512,7 @@ lilv_world_load_dyn_manifest(LilvWorld*      world,
 		// Get binary path
 		const SordNode* binary   = sord_iter_get_node(binaries, SORD_OBJECT);
 		const uint8_t*  lib_uri  = sord_node_get_string(binary);
-		const char*     lib_path = lilv_uri_to_path((const char*)lib_uri);
+		char*           lib_path = lilv_file_uri_parse((const char*)lib_uri, 0);
 		if (!lib_path) {
 			LILV_ERROR("No dynamic manifest library path\n");
 			sord_iter_free(binaries);
@@ -494,6 +526,7 @@ lilv_world_load_dyn_manifest(LilvWorld*      world,
 			LILV_ERRORF("Failed to open dynmanifest library `%s' (%s)\n",
 			            lib_path, dlerror());
 			sord_iter_free(binaries);
+			lilv_free(lib_path);
 			continue;
 		}
 
@@ -505,6 +538,7 @@ lilv_world_load_dyn_manifest(LilvWorld*      world,
 			LILV_ERRORF("No `lv2_dyn_manifest_open' in `%s'\n", lib_path);
 			sord_iter_free(binaries);
 			dlclose(lib);
+			lilv_free(lib_path);
 			continue;
 		}
 
@@ -517,6 +551,7 @@ lilv_world_load_dyn_manifest(LilvWorld*      world,
 			            lib_path);
 			sord_iter_free(binaries);
 			dlclose(lib);
+			lilv_free(lib_path);
 			continue;
 		}
 
@@ -563,6 +598,7 @@ lilv_world_load_dyn_manifest(LilvWorld*      world,
 		}
 		sord_iter_free(p);
 		sord_free(plugins);
+		lilv_free(lib_path);
 	}
 	sord_iter_free(iter);
 	sord_free(model);
@@ -570,7 +606,7 @@ lilv_world_load_dyn_manifest(LilvWorld*      world,
 }
 
 LilvNode*
-lilv_world_get_manifest_uri(LilvWorld* world, LilvNode* bundle_uri)
+lilv_world_get_manifest_uri(LilvWorld* world, const LilvNode* bundle_uri)
 {
 	SerdNode manifest_uri = lilv_new_uri_relative_to_base(
 		(const uint8_t*)"manifest.ttl",
@@ -580,8 +616,67 @@ lilv_world_get_manifest_uri(LilvWorld* world, LilvNode* bundle_uri)
 	return manifest;
 }
 
+static SordModel*
+load_plugin_model(LilvWorld*      world,
+                  const LilvNode* bundle_uri,
+                  const LilvNode* plugin_uri)
+{
+	// Create model and reader for loading into it
+	SordNode*   bundle_node = bundle_uri->node;
+	SordModel*  model       = sord_new(world->world, SORD_SPO|SORD_OPS, false);
+	SerdEnv*    env         = serd_env_new(sord_node_to_serd_node(bundle_node));
+	SerdReader* reader      = sord_new_reader(model, env, SERD_TURTLE, NULL);
+
+	// Load manifest
+	LilvNode* manifest_uri = lilv_world_get_manifest_uri(world, bundle_uri);
+	serd_reader_add_blank_prefix(reader, lilv_world_blank_node_prefix(world));
+	serd_reader_read_file(
+		reader, (const uint8_t*)lilv_node_as_string(manifest_uri));
+
+	// Load any seeAlso files
+	SordModel* files = lilv_world_filter_model(
+		world, model, plugin_uri->node, world->uris.rdfs_seeAlso, NULL, NULL);
+
+	SordIter* f = sord_begin(files);
+	FOREACH_MATCH(f) {
+		const SordNode* file      = sord_iter_get_node(f, SORD_OBJECT);
+		const uint8_t*  file_str  = sord_node_get_string(file);
+		if (sord_node_get_type(file) == SORD_URI) {
+			serd_reader_add_blank_prefix(
+				reader, lilv_world_blank_node_prefix(world));
+			serd_reader_read_file(reader, file_str);
+		}
+	}
+
+	sord_iter_free(f);
+	sord_free(files);
+	serd_reader_free(reader);
+	serd_env_free(env);
+	lilv_node_free(manifest_uri);
+
+	return model;
+}
+
+static LilvVersion
+get_version(LilvWorld* world, SordModel* model, const LilvNode* subject)
+{
+	const SordNode* minor_node = sord_get(
+		model, subject->node, world->uris.lv2_minorVersion, NULL, NULL);
+	const SordNode* micro_node = sord_get(
+		model, subject->node, world->uris.lv2_microVersion, NULL, NULL);
+
+
+	LilvVersion version = { 0, 0 };
+	if (minor_node && micro_node) {
+		version.minor = atoi((const char*)sord_node_get_string(minor_node));
+		version.micro = atoi((const char*)sord_node_get_string(micro_node));
+	}
+
+	return version;
+}
+
 LILV_API void
-lilv_world_load_bundle(LilvWorld* world, LilvNode* bundle_uri)
+lilv_world_load_bundle(LilvWorld* world, const LilvNode* bundle_uri)
 {
 	if (!lilv_node_is_uri(bundle_uri)) {
 		LILV_ERRORF("Bundle URI `%s' is not a URI\n",
@@ -606,6 +701,88 @@ lilv_world_load_bundle(LilvWorld* world, LilvNode* bundle_uri)
 	                                     world->uris.rdf_a,
 	                                     world->uris.lv2_Plugin,
 	                                     bundle_node);
+
+	// Find any loaded plugins that will be replaced with a newer version
+	LilvNodes* unload_uris = lilv_nodes_new();
+	FOREACH_MATCH(plug_results) {
+		const SordNode* plug = sord_iter_get_node(plug_results, SORD_SUBJECT);
+
+		LilvNode*         plugin_uri  = lilv_node_new_from_node(world, plug);
+		const LilvPlugin* plugin      = lilv_plugins_get_by_uri(world->plugins, plugin_uri);
+		const LilvNode*   last_bundle = plugin ? lilv_plugin_get_bundle_uri(plugin) : NULL;
+		if (!plugin || sord_node_equals(bundle_node, last_bundle->node)) {
+			// No previously loaded version, or it's from the same bundle
+			lilv_node_free(plugin_uri);
+			continue;
+		}
+
+		// Compare versions
+		SordModel*  this_model   = load_plugin_model(world, bundle_uri, plugin_uri);
+		LilvVersion this_version = get_version(world, this_model, plugin_uri);
+		SordModel*  last_model   = load_plugin_model(world, last_bundle, plugin_uri);
+		LilvVersion last_version = get_version(world, last_model, plugin_uri);
+		sord_free(this_model);
+		sord_free(last_model);
+		const int cmp = lilv_version_cmp(&this_version, &last_version);
+		if (cmp > 0) {
+			zix_tree_insert((ZixTree*)unload_uris,
+			                lilv_node_duplicate(plugin_uri),
+			                NULL);
+			LILV_WARNF("Replacing version %d.%d of <%s> from <%s>\n",
+			           last_version.minor, last_version.micro,
+			           sord_node_get_string(plug),
+			           sord_node_get_string(last_bundle->node));
+			LILV_NOTEF("New version %d.%d found in <%s>\n",
+			           this_version.minor, this_version.micro,
+			           sord_node_get_string(bundle_node));
+		} else if (cmp < 0) {
+			LILV_WARNF("Ignoring bundle <%s>\n",
+			           sord_node_get_string(bundle_node));
+			LILV_NOTEF("Newer version of <%s> loaded from <%s>\n",
+			           sord_node_get_string(plug),
+			           sord_node_get_string(last_bundle->node));
+			lilv_node_free(plugin_uri);
+			sord_iter_free(plug_results);
+			lilv_world_drop_graph(world, bundle_node);
+			lilv_node_free(manifest);
+			lilv_nodes_free(unload_uris);
+			return;
+		}
+		lilv_node_free(plugin_uri);
+	}
+
+	sord_iter_free(plug_results);
+
+	// Unload any old conflicting plugins
+	LilvNodes* unload_bundles = lilv_nodes_new();
+	LILV_FOREACH(nodes, i, unload_uris) {
+		const LilvNode*   uri    = lilv_nodes_get(unload_uris, i);
+		const LilvPlugin* plugin = lilv_plugins_get_by_uri(world->plugins, uri);
+		const LilvNode*   bundle = lilv_plugin_get_bundle_uri(plugin);
+
+		// Unload plugin and record bundle for later unloading
+		lilv_world_unload_resource(world, uri);
+		zix_tree_insert((ZixTree*)unload_bundles,
+		                lilv_node_duplicate(bundle),
+		                NULL);
+
+	}
+	lilv_nodes_free(unload_uris);
+
+	// Now unload the associated bundles
+	// This must be done last since several plugins could be in the same bundle
+	LILV_FOREACH(nodes, i, unload_bundles) {
+		lilv_world_unload_bundle(world, lilv_nodes_get(unload_bundles, i));
+	}
+	lilv_nodes_free(unload_bundles);
+
+	// Re-search for plugin results now that old plugins are gone
+	plug_results = sord_search(world->model,
+	                           NULL,
+	                           world->uris.rdf_a,
+	                           world->uris.lv2_Plugin,
+	                           bundle_node);
+
 	FOREACH_MATCH(plug_results) {
 		const SordNode* plug = sord_iter_get_node(plug_results, SORD_SUBJECT);
 		lilv_world_add_plugin(world, plug, manifest, NULL, bundle_node);
@@ -633,14 +810,14 @@ lilv_world_load_bundle(LilvWorld* world, LilvNode* bundle_uri)
 }
 
 static int
-lilv_world_drop_graph(LilvWorld* world, LilvNode* graph)
+lilv_world_drop_graph(LilvWorld* world, const SordNode* graph)
 {
-	SordIter* i = sord_search(world->model, NULL, NULL, NULL, graph->node);
+	SordIter* i = sord_search(world->model, NULL, NULL, NULL, graph);
 	while (!sord_iter_end(i)) {
 		const SerdStatus st = sord_erase(world->model, i);
 		if (st) {
 			LILV_ERRORF("Error removing statement from <%s> (%s)\n",
-			            lilv_node_as_uri(graph), serd_strerror(st));
+			            sord_node_get_string(graph), serd_strerror(st));
 			return st;
 		}
 	}
@@ -651,7 +828,7 @@ lilv_world_drop_graph(LilvWorld* world, LilvNode* graph)
 
 /** Remove loaded_files entry so file will be reloaded if requested. */
 static int
-lilv_world_unload_file(LilvWorld* world, LilvNode* file)
+lilv_world_unload_file(LilvWorld* world, const LilvNode* file)
 {
 	ZixTreeIter* iter;
 	if (!zix_tree_find((ZixTree*)world->loaded_files, file, &iter)) {
@@ -662,19 +839,54 @@ lilv_world_unload_file(LilvWorld* world, LilvNode* file)
 }
 
 LILV_API int
-lilv_world_unload_bundle(LilvWorld* world, LilvNode* bundle_uri)
+lilv_world_unload_bundle(LilvWorld* world, const LilvNode* bundle_uri)
 {
 	if (!bundle_uri) {
 		return 0;
 	}
 
-	// Remove loaded_files entry for manifest.ttl
-	LilvNode* manifest = lilv_world_get_manifest_uri(world, bundle_uri);
-	lilv_world_unload_file(world, manifest);
-	lilv_node_free(manifest);
+	// Find all loaded files that are inside the bundle
+	LilvNodes* files = lilv_nodes_new();
+	LILV_FOREACH(nodes, i, world->loaded_files) {
+		const LilvNode* file = lilv_nodes_get(world->loaded_files, i);
+		if (!strncmp(lilv_node_as_string(file),
+		             lilv_node_as_string(bundle_uri),
+		             strlen(lilv_node_as_string(bundle_uri)))) {
+			zix_tree_insert((ZixTree*)files,
+			                lilv_node_duplicate(file),
+			                NULL);
+		}
+	}
+
+	// Unload all loaded files in the bundle
+	LILV_FOREACH(nodes, i, files) {
+		const LilvNode* file = lilv_nodes_get(world->plugins, i);
+		lilv_world_unload_file(world, file);
+	}
+
+	lilv_nodes_free(files);
+
+	/* Remove any plugins in the bundle from the plugin list.  Since the
+	   application may still have a pointer to the LilvPlugin, it can not be
+	   destroyed here.  Instead, we move it to the zombie plugin list, so it
+	   will not be in the list returned by lilv_world_get_all_plugins() but can
+	   still be used.
+	*/
+	ZixTreeIter* i = zix_tree_begin((ZixTree*)world->plugins);
+	while (i != zix_tree_end((ZixTree*)world->plugins)) {
+		LilvPlugin*  p    = (LilvPlugin*)zix_tree_get(i);
+		ZixTreeIter* next = zix_tree_iter_next(i);
+
+		if (lilv_node_equals(lilv_plugin_get_bundle_uri(p), bundle_uri)) {
+			zix_tree_remove((ZixTree*)world->plugins, i);
+			zix_tree_insert((ZixTree*)world->zombies, p, NULL);
+		}
+
+		i = next;
+	}
 
 	// Drop everything in bundle graph
-	return lilv_world_drop_graph(world, bundle_uri);
+	return lilv_world_drop_graph(world, bundle_uri->node);
 }
 
 static void
@@ -907,7 +1119,7 @@ lilv_world_unload_resource(LilvWorld*      world,
 		if (sord_node_get_type(file) != SORD_URI) {
 			LILV_ERRORF("rdfs:seeAlso node `%s' is not a URI\n",
 			            sord_node_get_string(file));
-		} else if (!lilv_world_drop_graph(world, file_node)) {
+		} else if (!lilv_world_drop_graph(world, file_node->node)) {
 			lilv_world_unload_file(world, file_node);
 			++n_dropped;
 		}
@@ -936,3 +1148,51 @@ lilv_world_get_all_plugins(const LilvWorld* world)
 {
 	return world->plugins;
 }
+
+LILV_API LilvNode*
+lilv_world_get_symbol(LilvWorld* world, const LilvNode* subject)
+{
+	// Check for explicitly given symbol
+	SordNode* snode = sord_get(
+		world->model, subject->node, world->uris.lv2_symbol, NULL, NULL);
+
+	if (snode) {
+		LilvNode* ret = lilv_node_new_from_node(world, snode);
+		sord_node_free(world->world, snode);
+		return ret;
+	}
+
+	if (!lilv_node_is_uri(subject)) {
+		return NULL;
+	}
+
+	// Find rightmost segment of URI
+	SerdURI uri;
+	serd_uri_parse((const uint8_t*)lilv_node_as_uri(subject), &uri);
+	const char* str = "_";
+	if (uri.fragment.buf) {
+		str = (const char*)uri.fragment.buf + 1;
+	} else if (uri.query.buf) {
+		str = (const char*)uri.query.buf;
+	} else if (uri.path.buf) {
+		const char* last_slash = strrchr((const char*)uri.path.buf, '/');
+		str = last_slash ? (last_slash + 1) : (const char*)uri.path.buf;
+	}
+
+	// Replace invalid characters
+	const size_t len = strlen(str);
+	char* const  sym = (char*)calloc(1, len + 1);
+	for (size_t i = 0; i < len; ++i) {
+		const char c = str[i];
+		if (!((c >= 'a' && c <= 'z') || (c >= 'A' && c <= 'Z') ||
+		      (c == '_') || (i > 0 && c >= '0' && c <= '9'))) {
+			sym[i] = '_';
+		} else {
+			sym[i] = str[i];
+		}
+	}
+
+	LilvNode* ret = lilv_new_string(world, sym);
+	free(sym);
+	return ret;
+}
